from sqlalchemy import Column, String,ForeignKey, Integer
from sqlalchemy.orm import relationship
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy_utils import UUIDType, URLType
from sspo_db.config.base import Entity

class ApplicationType (Entity):
    
    __tablename__ = "application_type"
    application = relationship("Application", back_populates="application_type")
    
class Application(Entity):
    
    __tablename__ = "application"
    application_type_id = Column(Integer, ForeignKey('application_type.id'))
    application_type = relationship("ApplicationType", back_populates="application")

    configuration = relationship("Configuration", back_populates="application")
    
    def __str__(self):
        return self.name

class Configuration(Entity):
    
    __tablename__ = "configuration"

    secret = Column(String(200), nullable=False)
    url = Column(URLType)
    
    application_id = Column(Integer, ForeignKey('application.id'))
    application = relationship("Application", back_populates="configuration")
    organization_id = Column(Integer, ForeignKey('organization.id'))
    organization = relationship("Organization", back_populates="configuration") 


class ApplicationReference(Entity):

    __tablename__ = "application_reference"
    # external application's data
    application = Column(Integer, ForeignKey('application.id'))
    
    external_id = Column(String(200), nullable=False)
    external_url = Column(URLType)
    external_type_entity = Column(String(200), nullable=False)
    
    #Internal BD
    internal_uuid = Column(UUIDType(binary=False))
    entity_name = Column(String(200), nullable=False)



    


from sspo_db.model.stakeholder.models import Person, Developer, ScrumMaster
from sspo_db.service.stakeholder.service import PersonService, DeveloperService, ScrumMasterService
from sspo_db.application.core.application_application_reference import ApplicationApplicationReference

class ApplicationPerson():
    
    def __init__(self):
        self.person_service = PersonService()
        self.application_reference_service = ApplicationApplicationReference()
    
    def retrive_by_uuid(self, uuid):
        return self.person_service.get_by_uuid(uuid)
    
    def retrive_person_by_external_uuid (self, external_uuid):
        
        application_reference = self.application_reference_service.get_by_external_uuid(external_uuid)
        if application_reference:
            return self.person_service.get_by_uuid(application_reference.internal_uuid)
        return None

    def create_person(self, name, email, organization, description = ""):
        
        person = Person()
        person.name = name
        person.description = description
        person.email = email 
        person.organization = organization

        self.person_service.create(person)

        return person
        

class ApplicationStakeholder():

    def __init__(self):
        self.developer_service = DeveloperService()
        self.scrum_master_service = ScrumMasterService()
    
    def create_developer(self, person, team, team_role = ""):
        
        developer = Developer()
        developer.name = team.name+" - "+person.name
        developer.description = person.description
        developer.person = person
        developer.team = team 
        developer.team_role = team_role

        self.developer_service.create(developer)

    def create_scrum_master(self, person, team, team_role = ""):
        
        scrum_master = ScrumMaster()
        scrum_master.name = team.name+" - "+person.name
        scrum_master.description = person.description
        scrum_master.person = person
        scrum_master.team = team 
        scrum_master.team_role = team_role

        self.scrum_master_service.create(scrum_master)
    
    


from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sspo_db.model.organization.models import *
from sspo_db.service.base_service import BaseService

class OrganizationService(BaseService):
    def __init__(self):
        super(OrganizationService,self).__init__(Organization)

class TeamService(BaseService):
    def __init__(self):
        super(TeamService,self).__init__(Team)

class ScrumTeamService(BaseService):
    def __init__(self):
        super(ScrumTeamService,self).__init__(ScrumTeam)

class DevelopmentTeamService(BaseService):
    def __init__(self):
        super(DevelopmentTeamService,self).__init__(DevelopmentTeam)



from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sspo_db.model.process.models import *
from sspo_db.service.base_service import BaseService

class ScrumProjectService(BaseService):
    def __init__(self):
        super(ScrumProjectService,self).__init__(ScrumProject)

class ScrumComplexProjectService(BaseService):
    def __init__(self):
        super(ScrumComplexProjectService,self).__init__(ScrumComplexProject)

class ScrumAtomicProjectService(BaseService):
    def __init__(self):
        super(ScrumAtomicProjectService,self).__init__(ScrumAtomicProject)

class ScrumProcessService(BaseService):
    def __init__(self):
        super(ScrumProcessService,self).__init__(ScrumProcess)

class ProductBacklogDefinitionService(BaseService):
    def __init__(self):
        super(ProductBacklogDefinitionService,self).__init__(ProductBacklogDefinition)

class SprintService(BaseService):
    def __init__(self):
        super(SprintService,self).__init__(Sprint)

class CerimonyService(BaseService):
    def __init__(self):
        super(CerimonyService,self).__init__(Cerimony)

class PlanningMeetingService(BaseService):
    def __init__(self):
        super(PlanningMeetingService,self).__init__(PlanningMeeting)

class DailyStandupMeetingService(BaseService):
    def __init__(self):
        super(DailyStandupMeetingService,self).__init__(DailyStandupMeeting)

class ReviewMeetingService(BaseService):
    def __init__(self):
        super(ReviewMeetingService,self).__init__(ReviewMeeting)

class RetrospectiveMeetingService(BaseService):
    def __init__(self):
        super(RetrospectiveMeetingService,self).__init__(RetrospectiveMeeting)


from sspo_db.config.base import Entity
from sspo_db.config.config import Base
from sqlalchemy import Column, Boolean ,ForeignKey, Integer, DateTime, String, Table
from sqlalchemy.orm import relationship

association_sprint_scrum_development_table = Table('association_sprint_scrum_development', Base.metadata,
    Column('scrum_development_task_id', Integer, ForeignKey('scrum_development_task.id')),
    Column('sprint_id', Integer, ForeignKey('sprint.id'))
)

class ScrumProject(Entity):
    is_instance_of = "spo.software_project"
    __tablename__ = "scrum_project"

    root = Column(Boolean, unique=False, default=True)
    organization_id = Column(Integer, ForeignKey('organization.id'))
    type = Column(String(50))

    organization = relationship("Organization", back_populates="scrum_project")
    scrum_process = relationship("ScrumProcess", back_populates="scrum_project", uselist=False)

    #scrum_complex_project_id = Column(Integer, ForeignKey('scrum_project.id'))
    #scrum_complex_project = relationship("ScrumComplexProject",back_populates="scrum_projects", foreign_keys=[scrum_complex_project_id])

    __mapper_args__ = {
        'polymorphic_identity':'scrum_project',
        'polymorphic_on':type
    }
    
class ScrumComplexProject(ScrumProject):
    is_instance_of = "spo.software_project.complex"  
    __tablename__ = "scrum_complex_project"

    id = Column(Integer, ForeignKey('scrum_project.id'), primary_key=True)

    #scrum_projects = relationship("ScrumProject", back_populates="scrum_complex_project")

    __mapper_args__ = {
        'polymorphic_identity':'scrum_complex_project',
    }


class ScrumAtomicProject(ScrumProject):
    is_instance_of = "spo.software_project.atomic"
    __tablename__ = "scrum_atomic_project"

    id = Column(Integer, ForeignKey('scrum_project.id'), primary_key=True)
    
    scrum_complex_project_id = Column(Integer, ForeignKey('scrum_complex_project.id'))
    
    __mapper_args__ = {
        'polymorphic_identity':'scrum_atomic_project',
    }

class ScrumProcess(Entity):
    is_instance_of = "spo.performed.process.general.project"
    __tablename__ = "scrum_process"
    
    scrum_project_id = Column(Integer, ForeignKey('scrum_project.id'))
    scrum_project= relationship("ScrumProject", back_populates="scrum_process")

    product_backlog_definition = relationship("ProductBacklogDefinition", back_populates="scrum_process")
    sprint = relationship("Sprint", back_populates="scrum_process")

class ProductBacklogDefinition(Entity):
    is_instance_of = "spo.performed.process.specific.project"
    __tablename__ = "product_backlog_definition"

    scrum_process_id = Column(Integer, ForeignKey('scrum_process.id'))
    scrum_process= relationship("ScrumProcess", back_populates="product_backlog_definition")

class Sprint(Entity):

    is_instance_of = "spo.performed.process.specific.project"
    __tablename__ = "sprint"
    
    startDate  = Column(DateTime)
    endDate = Column(DateTime)
    
    scrum_process_id = Column(Integer, ForeignKey('scrum_process.id'))
    scrum_process = relationship("ScrumProcess", back_populates="sprint")
    
    scrum_development_tasks = relationship("ScrumDevelopmentTask", secondary=association_sprint_scrum_development_table, back_populates="sprints")
    cerimony = relationship("Cerimony", back_populates="sprint")
    
class Cerimony(Entity):
    is_instance_of = "spo.performed.activity.project"
    __tablename__ = "cerimony"

    startDate  = Column(DateTime)
    endDate = Column(DateTime)

    sprint_id = Column(Integer, ForeignKey('sprint.id'))
    sprint = relationship("Sprint", back_populates="cerimony")
    
    type = Column(String(50))

    __mapper_args__ = {
        'polymorphic_identity':'cerimony',
        'polymorphic_on':type
    }

   
class PlanningMeeting(Cerimony):
    __tablename__ = "planning_meeting"
    
    id = Column(Integer, ForeignKey('cerimony.id'), primary_key=True)
    __mapper_args__ = {
        'polymorphic_identity':'planning_meeting',
    }

   
class DailyStandupMeeting(Cerimony):
    __tablename__ = "daily_standup_meeting"
    
    id = Column(Integer, ForeignKey('cerimony.id'), primary_key=True)
    
    __mapper_args__ = {
        'polymorphic_identity':'daily_standup_meeting',
    }


class ReviewMeeting(Cerimony):
    __tablename__ = "review_meeting"

    id = Column(Integer, ForeignKey('cerimony.id'), primary_key=True)
    __mapper_args__ = {
        'polymorphic_identity':'review_meeting',
    }


class RetrospectiveMeeting(Cerimony):
    __tablename__ = "retrospective_meeting"

    id = Column(Integer, ForeignKey('cerimony.id'), primary_key=True)
    
    __mapper_args__ = {
        'polymorphic_identity':'retrospective_meeting',
    }



from sspo_db.service.artifact.service import EpicService

class ApplicationEpic():
    
    def __init__(self):
        self.service = EpicService()
    
    def create (self, epic):
        self.service.create (epic)
        return epic
        


from sspo_db.model.process.models import Sprint
from sspo_db.service.process.service import SprintService, ScrumAtomicProjectService
from sspo_db.application.core.application_application_reference import ApplicationApplicationReference

class ApplicationSprint():

    def __init__(self):
        self.sprint_service = SprintService()
    
    def create (self, name, start_date, end_date, scrum_process):
        
        sprint = Sprint()
        sprint.name = name
        sprint.description = name
        sprint.startDate = start_date
        sprint.endDate = end_date
        sprint.scrum_process = scrum_process
        
        self.sprint_service.create(sprint)
        
        return sprint

class ApplicationScrumAtomic():
    
    def __init__(self):
        self.scrum_atomic_project_service = ScrumAtomicProjectService()
        self.application_application_reference = ApplicationApplicationReference()
    
    def retrive_by_external_uuid(self, external_uuid):
        application_reference = self.application_application_reference.get_by_external_uuid_and_seon_entity_name(external_uuid, "scrum_atomic_project")
        if application_reference:
            return self.scrum_atomic_project_service.get_by_uuid(application_reference.internal_uuid)
        return None
